﻿using Microsoft.Win32;
using System.IO;
using System.Text;
using System.Windows;
using System.Windows.Documents;
using System.Windows.Markup;
using System.Xml;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 파일 열기 대화 상자
        /// </summary>
        private OpenFileDialog openFileDialog;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            this.openFileDialog = new OpenFileDialog();

            this.openFileDialog.Filter      = "RTF 파일 (*.rtf)|*.rtf";
            this.openFileDialog.FilterIndex = 1;
            this.openFileDialog.DefaultExt  = ".rtf";

            this.openFileButton.Click += openFileButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 파일 열기 버튼 클릭시 처리하기 - openFileButton_Click(sender, e)

        /// <summary>
        /// 파일 열기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void openFileButton_Click(object sender, RoutedEventArgs e)
        {
            bool result = this.openFileDialog.ShowDialog(this).GetValueOrDefault();

            if(result)
            {
                string rtf = File.ReadAllText(this.openFileDialog.FileName, Encoding.UTF8);

                string xaml = GetXAML(rtf);

                FlowDocument document = GetDocument(xaml);

                this.flowDocumentScrollViewer.Document = document;

                this.textBox.Text = xaml;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region XAML 구하기 - GetXAML(rtf)

        /// <summary>
        /// XAML 구하기
        /// </summary>
        /// <param name="rtf">RTF</param>
        /// <returns>XAML</returns>
        private string GetXAML(string rtf)
        {
            FlowDocument document = new FlowDocument();

            TextRange range = new TextRange(document.ContentStart, document.ContentEnd);

            using(MemoryStream sourceStream = new MemoryStream(Encoding.UTF8.GetBytes(rtf)))
            {
                range.Load(sourceStream, DataFormats.Rtf);

                using(MemoryStream targetStream = new MemoryStream())
                {
                    range.Save(targetStream, DataFormats.Xaml);

                    targetStream.Position = 0;

                    using(StreamReader reader = new StreamReader(targetStream))
                    {
                        string xaml = reader.ReadToEnd();

                        return xaml;
                    }
                }
            }
        }

        #endregion
        #region 문서 구하기 - GetDocument(xaml)

        /// <summary>
        /// 문서 구하기
        /// </summary>
        /// <param name="xaml">XAML</param>
        /// <returns>문서</returns>
        private FlowDocument GetDocument(string xaml)
        {
            StringReader stringReader = new StringReader(xaml);

            XmlReader xmlReader = XmlReader.Create(stringReader);

            Section section = XamlReader.Load(xmlReader) as Section;

            FlowDocument document = new FlowDocument();

            while(section.Blocks.Count > 0)
            {
                Block block = section.Blocks.FirstBlock;

                section.Blocks.Remove(block);

                document.Blocks.Add(block);
            }

            return document;
        }

        #endregion
    }
}